local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local addon = _G[addonName];

--- Localize Vars
local Objects = addon.Core.Objects;

-- Objects
local Player = addon.Units.Player;
local Target = addon.Units.Target;
local Racial, Spell, Talent, Aura, Azerite, Item, Consumable;

-- Rotation Variables
local nameAPL = "huntsthewind_monk_windwalker";

-- CD Rotation
local function Cooldowns(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- actions.cd+=/arcane_torrent,if=chi.max-chi>=1&energy.time_to_max>=0.5
        ArcaneTorrent = function()
            return Player.Chi.Deficit() >= 1
                    and Player.Energy.TimeToMax() >= 0.5;
        end,

        TouchOfDeath = {
            -- actions.cd+=/touch_of_death,target_if=min:dot.touch_of_death.remains,if=((talent.serenity.enabled&cooldown.serenity.remains<=1)&cooldown.fists_of_fury.remains<=4)&cooldown.rising_sun_kick.remains<7&!prev_gcd.1.touch_of_death
            -- ignore target_if clause
            WithSerenity = function()
                return ((Talent.Serenity.Enabled()
                        and Talent.Serenity.Cooldown.Remains() <= 1)
                        and Spell.FistsOfFury.Cooldown.Remains() <= 4)
                        and Spell.RisingSunKick.Cooldown.Remains() < 7
                        and not Player.PrevGCD(1, Spell.TouchOfDeath);
            end,

            -- actions.cd+=/touch_of_death,target_if=min:dot.touch_of_death.remains,if=((!talent.serenity.enabled&cooldown.storm_earth_and_fire.remains<=1)|chi>=2)&cooldown.fists_of_fury.remains<=4&cooldown.rising_sun_kick.remains<7&!prev_gcd.1.touch_of_death
            -- ignore target_if clause
            WithStormEarthAndFire = function()
                return ((not Talent.Serenity.Enabled()
                        and Spell.StormEarthAndFire.Enabled()
                        and Spell.StormEarthAndFire.Cooldown.Remains() <= 1)
                        or Player.Chi() >= 2)
                        and Spell.FistsOfFury.Cooldown.Remains() <= 4
                        and Spell.RisingSunKick.Cooldown.Remains() < 7
                        and not Player.PrevGCD(1, Spell.TouchOfDeath);
            end,
        },
    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        -- actions.cd=invoke_xuen_the_white_tiger
        action.EvaluateAction(Talent.InvokeXuenTheWhiteTiger, true);
        -- actions.cd+=/blood_fury
        action.EvaluateAction(Racial.BloodFury, true);
        -- actions.cd+=/berserking
        action.EvaluateAction(Racial.Berserking, true);
        action.EvaluateAction(Racial.ArcaneTorrent, self.Requirements.ArcaneTorrent);
        -- actions.cd+=/lights_judgement
        -- action.EvaluateAction(Racial.LightsJudgement, true);
        action.EvaluateAction(Spell.TouchOfDeath, self.Requirements.TouchOfDeath.WithSerenity);
        action.EvaluateCycleAction(Spell.TouchOfDeath, self.Requirements.TouchOfDeath.WithStormEarthAndFire);
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationCooldowns = Cooldowns("Cooldowns");

-- AOE Rotation
local function AOE(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- actions.aoe+=/arcane_torrent,if=chi.max-chi>=1&energy.time_to_max>=0.5
        ArcaneTorrent = function()
            return Player.Chi.Deficit() >= 1
            and Player.Energy.TimeToMax() >= 0.5;
        end,

        -- actions.aoe+=/blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(chi>1|buff.bok_proc.up|(talent.energizing_elixir.enabled&cooldown.energizing_elixir.remains<cooldown.fists_of_fury.remains))&((cooldown.rising_sun_kick.remains>1&(!talent.fist_of_the_white_tiger.enabled|cooldown.fist_of_the_white_tiger.remains>1)|chi>4)&(cooldown.fists_of_fury.remains>1|chi>2)|prev_gcd.1.tiger_palm)&!prev_gcd.1.blackout_kick
        -- Removed lines that used previous tier bonuses
        -- Ignore the target_if clause, since we can't do it
        BlackoutKick = function()
            return (Player.Chi() > 1
                    or Player.Buff(Aura.BlackoutKick).Up()
                    or (Talent.EnergizingElixir.Enabled()
                        and Talent.EnergizingElixir.Cooldown.Remains() < Spell.FistsOfFury.Cooldown.Remains()))
                and ((((Spell.RisingSunKick.Cooldown.Remains() > 1
                        and (not Talent.FistOfTheWhiteTiger.Enabled()
                            or Talent.FistOfTheWhiteTiger.Cooldown.Remains() > 1))
                        or Player.Chi() > 4)
                    and (Spell.FistsOfFury.Cooldown.Remains() > 1
                        or Player.Chi() > 2))
                    or Player.PrevGCD(1, Spell.TigerPalm))
                and Player.PrevGCD(1, Spell.BlackoutKick);
        end,

        -- actions.aoe+=/chi_burst,if=chi<=3&(cooldown.rising_sun_kick.remains>=5|cooldown.whirling_dragon_punch.remains>=5)&energy.time_to_max>1
        -- LUNA: do I need to check for talents to be enabled, or do you handle
        --       that upstream?
        ChiBurst = function()
            return Talent.ChiBurst.Enabled()
                and Player.Chi() <= 3
                and (Spell.RisingSunKick.Cooldown.Remains() >= 5
                    or (Talent.WhirlingDragonPunch.Enabled()
                        and Talent.WhirlingDragonPunch.Cooldown.Remains() >= 5))
                and Player.Energy.TimeToMax() > 1;
        end,

        -- actions.aoe+=/chi_wave,if=chi<=3&(cooldown.rising_sun_kick.remains>=5|cooldown.whirling_dragon_punch.remains>=5)&energy.time_to_max>1
        ChiWave = function()
            return Player.Chi() <= 3
                and (Spell.RisingSunKick.Cooldown.Remains() >= 5
                    or (Talent.WhirlingDragonPunch.Enabled()
                        and Talent.WhirlingDragonPunch.Cooldown.Remains() >= 5))
                and Player.Energy.TimeToMax() > 1;
        end,

        -- actions.aoe+=/energizing_elixir,if=!prev_gcd.1.tiger_palm&chi<=1&(cooldown.rising_sun_kick.remains=0|(talent.fist_of_the_white_tiger.enabled&cooldown.fist_of_the_white_tiger.remains=0)|energy<50)
        EnergizingElixir = function()
            return not Player.PrevGCD(1, Spell.TigerPalm)
                and Player.Chi() <= 1
                and (Spell.RisingSunKick.Cooldown.Remains() == 0
                    or (Talent.FistOfTheWhiteTiger.Enabled()
                        and Talent.FistOfTheWhiteTiger.Cooldown.Remains() == 0)
                    or Player.Energy() < 50);
        end,

        FistsOfFury = {
            -- actions.aoe+=/fists_of_fury,if=cooldown.rising_sun_kick.remains>=3.5&chi<=5
            Use = function()
                return Spell.RisingSunKick.Cooldown.Remains() >= 3.5
                    and Player.Chi() <= 5;
            end,

            -- took out special handling for Drinking Horn
            -- actions.aoe+=/fists_of_fury,if=talent.serenity.enabled&cooldown.serenity.remains>=5&energy.time_to_max>2
            WithSerenity = function()
                return Talent.Serenity.Enabled()
                    and Talent.Serenity.Cooldown.Remains() >= 5
                    and Player.Energy.TimeToMax() > 2;
            end,

            -- actions.aoe+=/fists_of_fury,if=!talent.serenity.enabled&energy.time_to_max>2
            WithoutSerenity = function()
                return not Talent.Serenity.Enabled()
                    and Player.Energy.TimeToMax() > 2;
            end,
        },

        -- actions.aoe+=/rising_sun_kick,target_if=cooldown.whirling_dragon_punch.remains>=gcd&!prev_gcd.1.rising_sun_kick&cooldown.fists_of_fury.remains>gcd
        -- LUNA: hmm, what if player doesn't take WDP?
        RisingSunKick = function()
            return Talent.WhirlingDragonPunch.Enabled()
                and Talent.WhirlingDragonPunch.Cooldown.Remains() >= Player.GCD()
                and not Player.PrevGCD(1, Spell.RisingSunKick)
                and Spell.FistsOfFury.Cooldown.Remains() > Player.GCD();
        end,

        -- actions.aoe+=/spinning_crane_kick,if=active_enemies>=3&!prev_gcd.1.spinning_crane_kick
        -- Removed special case where player has Tier 21 4pc
        SpinningCraneKick = function(numEnemies)
            return numEnemies >= 3
                and not Player.PrevGCD(1, Spell.SpinningCraneKick);
        end,

        TigerPalm = {
            -- actions.aoe+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy.time_to_max<=1&chi.max-chi>=2
            -- ignore target_if clause since we can't do it
            MaxEnergy = function()
                return not Player.PrevGCD(1, Spell.TigerPalm)
                    and not Player.PrevGCD(1, Talent.EnergizingElixir)
                    and Player.Energy.TimeToMax() <= 1
                    and Player.Chi.Deficit() >= 2;
            end,

            -- actions.aoe+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&(chi.max-chi>=2|energy.time_to_max<3)
            Use = function()
                return not Player.PrevGCD(1, Spell.TigerPalm)
                    and not Player.PrevGCD(1, Talent.EnergizingElixir)
                    and (Player.Chi.Deficit() >= 2
                        or Player.Energy.TimeToMax() < 3);
            end,
        },

    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        -- Simcraft file says this still needs to be optimized
        action.CallActionList(rotationCooldowns);
        action.EvaluateAction(Talent.EnergizingElixir, self.Requirements.EnergizingElixir);
        action.EvaluateAction(Racial.ArcaneTorrent, self.Requirements.ArcaneTorrent);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury.WithSerenity);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury.WithoutSerenity);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury.Use);
        action.EvaluateAction(Talent.WhirlingDragonPunch, true);
        action.EvaluateAction(Spell.RisingSunKick, self.Requirements.RisingSunKick);
        action.EvaluateAction(Talent.ChiBurst, self.Requirements.ChiBurst);
        action.EvaluateAction(Talent.ChiBurst, true);
        action.EvaluateAction(Spell.SpinningCraneKick, self.Requirements.SpinningCraneKick);
        action.EvaluateAction(Spell.BlackoutKick, self.Requirements.BlackoutKick);
        action.EvaluateAction(Spell.TigerPalm, self.Requirements.TigerPalm.Use);
        action.EvaluateAction(Spell.TigerPalm, self.Requirements.TigerPalm.MaxEnergy);
        action.EvaluateAction(Talent.ChiWave, self.Requirements.ChiWave);
        action.EvaluateAction(Talent.ChiWave, true);
    end

    function self.Use(numEnemies)
        return numEnemies > 3
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationAOE = AOE("AOE");

-- Serenity Rotation
local function Serenity(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        BlackoutKick = {
            -- actions.serenity+=/blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&cooldown.rising_sun_kick.remains>=2&cooldown.fists_of_fury.remains>=2
            CooldownsActive = function()
                return not Player.PrevGCD(1, Spell.BlackoutKick)
                    and Spell.RisingSunKick.Cooldown.Remains() >= 2
                    and Spell.FistsOfFury.Cooldown.Remains() >= 2;
            end,

            -- actions.serenity+=/blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick
            Use = function()
                return not Player.PrevGCD(1, Spell.BlackoutKick);
            end,
        },

        FistsOfFury = {
            -- actions.serenity+=/fists_of_fury,if=prev_gcd.1.rising_sun_kick&prev_gcd.2.serenity
            AfterRisingSunKickAndSerenity = function()
                return Player.PrevGCD(1, Spell.RisingSunKick)
                    and Player.PrevGCD(2, Talent.Serenity);
            end,

            -- # Cast Fist of Fury if:
            -- # - The remaining cooldown on rising_sun_kick is longer than the channel duration of Fists_of_fury
            -- actions.serenity+=/fists_of_fury,if=((!equipped.drinking_horn_cover|buff.bloodlust.up|buff.serenity.remains<1)&(cooldown.rising_sun_kick.remains>1|active_enemies>1)),interrupt=1
            Use = function(numEnemies)
                return (Player.HasBloodlust() or Player.Buff(Aura.Serenity).Remains() < 1)
                   and (Spell.RisingSunKick.Cooldown.Remains() > 1 or numEnemies > 1);
            end,

            -- interrupt=1
            Interrupt = function()
                return true;
            end,
        },

        -- actions.serenity=fist_of_the_white_tiger,if=buff.bloodlust.up&!buff.serenity.up
        FistOfTheWhiteTiger = function()
            return Player.HasBloodlust()
                and not Player.Buff(Aura.Serenity).Up();
        end,

        RisingSunKick = {
            -- actions.serenity+=/rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies>=3
            Use = function(numEnemies)
                return numEnemies >= 3;
            end,

            -- actions.serenity+=/rising_sun_kick,cycle_targets=1,if=active_enemies<3
            Cleave = function(numEnemies)
                return numEnemies < 3;
            end,
        },

        -- actions.serenity+=/rushing_jade_wind,if=talent.rushing_jade_wind.enabled&!prev_gcd.1.rushing_jade_wind&buff.rushing_jade_wind.down
        RushingJadeWind = function()
            return Talent.RushingJadeWind.Enabled()
                and not Player.PrevGCD(1, Talent.RushingJadeWind)
                and Player.Buff(Aura.RushingJadeWind).Down();
        end,

        SpinningCraneKick = {
            -- actions.serenity+=/spinning_crane_kick,if=!prev_gcd.1.spinning_crane_kick
            Use = function()
                return not Player.PrevGCD(1, Spell.SpinningCraneKick);
            end,

            -- actions.serenity+=/spinning_crane_kick,if=active_enemies>=3&!prev_gcd.1.spinning_crane_kick
            AOE = function(numEnemies)
                return numEnemies >= 3
                   and not Player.PrevGCD(1, Spell.SpinningCraneKick);
            end,
        },

        -- actions.serenity+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1&!buff.serenity.up
        -- ignore target_if since addon cannot do it
        TigerPalm = function()
            return not Player.PrevGCD(1, Spell.TigerPalm)
                and not Player.PrevGCD(1, Talent.EnergizingElixir)
                and Player.Energy() == Player.Energy.Max()
                and Player.Chi() < 1
                and not Player.Buff(Aura.Serenity).Up();
        end,
    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        action.EvaluateAction(Talent.FistOfTheWhiteTiger, self.Requirements.FistOfTheWhiteTiger);
        action.EvaluateAction(Spell.TigerPalm, self.Requirements.TigerPalm);
        -- actions.serenity+=/call_action_list,name=cd
        action.CallActionList(rotationCooldowns);
        action.EvaluateAction(Talent.RushingJadeWind, self.Requirements.RushingJadeWind);
        -- actions.serenity+=/serenity
        action.EvaluateAction(Talent.Serenity, true);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury.AfterRisingSunKickAndSerenity);
        action.EvaluateAction(Spell.BlackoutKick, self.Requirements.BlackoutKick.CooldownsActive);
        action.EvaluateInterruptCondition(Spell.FistsOfFury, self.Requirements.FistsOfFury.Use, self.Requirements.FistsOfFury.Interrupt);
        action.EvaluateAction(Spell.SpinningCraneKick, self.Requirements.SpinningCraneKick.AOE);
        action.EvaluateAction(Spell.RisingSunKick, self.Requirements.RisingSunKick.Use);
        action.EvaluateAction(Spell.SpinningCraneKick, self.Requirements.SpinningCraneKick.Use);
        action.EvaluateAction(Spell.BlackoutKick, self.Requirements.BlackoutKick.Use);
    end

    -- actions+=/call_action_list,name=serenity,if=(talent.serenity.enabled&cooldown.serenity.remains<=0)|buff.serenity.up
    function self.Use()
        return (Talent.Serenity.Enabled() and Talent.Serenity.Cooldown.Remains() <= 0)
            or Player.Buff(Aura.Serenity).Up();
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationSerenity = Serenity("Serenity");

-- Standard Rotation
local function Standard(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        BlackoutKick = {
            -- actions.st+=/blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1
            BuildChi = function()
                return not Player.PrevGCD(1, Spell.BlackoutKick)
                        and Player.Chi.Deficit() >= 1
            end,
        },

        ChiBurst = {
            -- actions.st+=/chi_burst,if=energy.time_to_max>1&talent.serenity.enabled
            NoCap = function()
                return Player.Energy.TimeToMax() > 1
                        and Talent.Serenity.Enabled();
            end,

            -- actions.st+=/chi_burst,if=chi.max-chi>=3&energy.time_to_max>1&!talent.serenity.enabled
            Use = function()
                return Player.Chi.Deficit() >= 3
                        and Player.Energy.TimeToMax() > 1
                        and not Talent.Serenity.Enabled();
            end,
        },

        -- actions.st+=/energizing_elixir,if=!prev_gcd.1.tiger_palm
        EnergizingElixir = function()
            return not Player.PrevGCD(1, Spell.TigerPalm);
        end,

        FistsOfFury = {
            -- actions.st+=/fists_of_fury,if=!talent.serenity.enabled&energy.time_to_max>2
            Use = function()
                return not Talent.Serenity.Enabled()
                        and Player.Energy.TimeToMax() > 2;
            end,

            -- actions.st+=/fists_of_fury,if=!talent.serenity.enabled
            NoSerenity = function()
                return not Talent.Serenity.Enabled();
            end,

            -- actions.st+=/fists_of_fury,if=talent.serenity.enabled&cooldown.serenity.remains>=5&energy.time_to_max>2
            WithSerenity = function()
                return Talent.Serenity.Enabled()
                        and Talent.Serenity.Cooldown.Remains() >= 5
                        and Player.Energy.TimeToMax() > 2;
            end,
        },

        -- actions.st+=/fist_of_the_white_tiger,if=chi.max-chi>=3
        FistOfTheWhiteTiger = function()
            return Player.Chi.Deficit() >= 3;
        end,

        RisingSunKick = {
            -- actions.st+=/rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=cooldown.serenity.remains>=5|(!talent.serenity.enabled)
            Use = function()
                return not Talent.Serenity.Enabled()
                        or Talent.Serenity.Cooldown.Remains() >= 5;
            end,

            -- actions.st+=/rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=((chi>=3&energy>=40)|chi>=5)&(talent.serenity.enabled|cooldown.serenity.remains>=6)
            MaxResources = function()
                return ((Player.Chi() >= 3 and Player.Energy() >= 40) or Player.Chi() >= 5)
                        and (not Talent.Serenity.Enabled() or Talent.Serenity.Cooldown.Remains() >= 6);
            end,
        },

        -- actions.st+=/rushing_jade_wind,if=buff.rushing_jade_wind.down&!prev_gcd.1.rushing_jade_wind
        RushingJadeWind = function()
            return not Player.Buff(Aura.RushingJadeWind).Up()
                    and not Player.PrevGCD(1, Talent.RushingJadeWind);
        end,

        -- actions.st+=/storm_earth_and_fire,if=!buff.storm_earth_and_fire.up
        StormEarthAndFire = function()
            return not Player.Buff(Aura.StormEarthAndFire).Up();
        end,

        TigerPalm = {
            -- actions.st+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&(chi.max-chi>=2|energy.time_to_max<3)&!buff.serenity.up
            Use = function()
                return not Player.PrevGCD(1, Spell.TigerPalm)
                        and not Player.PrevGCD(1, Talent.EnergizingElixir)
                        and (Player.Chi.Deficit() >= 2 or Player.Energy.TimeToMax() < 3)
                        and not Player.Buff(Aura.Serenity).Up();
            end,

            -- actions.st+=/tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy.time_to_max<=1&chi.max-chi>=2&!buff.serenity.up
            MaxEnergy = function()
                return not Player.PrevGCD(1, Spell.TigerPalm)
                        and not Player.PrevGCD(1, Talent.EnergizingElixir)
                        and Player.Energy.TimeToMax() <= 1
                        and Player.Chi.Deficit() >= 2
                        and not Player.Buff(Aura.Serenity).Up();
            end,
        },
    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        action.EvaluateAction(Talent.InvokeXuenTheWhiteTiger, true);
        action.EvaluateAction(Spell.StormEarthAndFire, self.Requirements.StormEarthAndFire);
        action.EvaluateAction(Talent.RushingJadeWind, self.Requirements.RushingJadeWind);
        action.EvaluateAction(Talent.EnergizingElixir, self.Requirements.EnergizingElixir);
        action.EvaluateCycleAction(Spell.TigerPalm, self.Requirements.TigerPalm.MaxEnergy);
        action.EvaluateAction(Talent.FistOfTheWhiteTiger, self.Requirements.FistOfTheWhiteTiger);
        -- actions.st+=/whirling_dragon_punch
        action.EvaluateAction(Talent.WhirlingDragonPunch, true);
        action.EvaluateCycleAction(Spell.RisingSunKick, self.Requirements.RisingSunKick.MaxResources);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury.WithSerenity);
        action.EvaluateAction(Spell.FistsOfFury, self.Requirements.FistsOfFury.NoSerenity);
        action.EvaluateCycleAction(Spell.RisingSunKick, self.Requirements.RisingSunKick.Use);
        action.EvaluateCycleAction(Spell.BlackoutKick, self.Requirements.BlackoutKick.BuildChi);
        action.EvaluateAction(Spell.BlackoutKick, true);
        action.EvaluateAction(Talent.ChiWave, true);
        action.EvaluateAction(Talent.ChiBurst, self.Requirements.ChiBurst.NoMax);
        action.EvaluateCycleAction(Spell.TigerPalm, self.Requirements.TigerPalm.Use);
        action.EvaluateAction(Talent.ChiBurst, self.Requirements.ChiBurst.Use);
    end

    function self.Use(numEnemies)
        return numEnemies <= 3
    end

    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationStandard = Standard("Standard");

-- Storm, Earth, and Fire Rotation
local function StormEarthAndFire(rotationName)
    -- Inherits Rotation Class so get the base class.
    local self = addonTable.rotationsClass(nameAPL, rotationName);

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- actions.sef+=/call_action_list,name=aoe,if=active_enemies>3
        AOE = function(numEnemies)
            return numEnemies > 3;
        end,

        -- actions.sef+=/call_action_list,name=st,if=active_enemies<=3
        Standard = function(numEnemies)
            return numEnemies <= 3;
        end,

        -- actions.sef+=/storm_earth_and_fire,if=!buff.storm_earth_and_fire.up
        StormEarthAndFire = function()
            return not Player.Buff(Aura.StormEarthAndFire).Up();
        end,

        -- actions.sef=tiger_palm,target_if=debuff.mark_of_the_crane.down,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1
        -- ignore target_if since addon cannot do it
        TigerPalm = function()
            return not Player.PrevGCD(1, Spell.TigerPalm)
                and not Player.PrevGCD(1, Talent.EnergizingElixir)
                and Player.Energy() == Player.Energy.Max()
                and Player.Chi() < 1;
        end,
    };

    Objects.FinalizeRequirements(self.Requirements);

    function self.Rotation(action)
        action.EvaluateCycleAction(Spell.TigerPalm, self.Requirements.TigerPalm);
        -- actions.sef+=/call_action_list,name=cd
        action.CallActionList(rotationCooldowns);
        action.EvaluateCycleAction(Spell.StormEarthAndFire, self.Requirements.StormEarthAndFire);
        action.CallActionList(rotationAOE, self.Requirements.AOE);
        action.CallActionList(rotationStandard, self.Requirements.Standard);
    end

    -- actions+=/call_action_list,name=sef,if=!talent.serenity.enabled&(buff.storm_earth_and_fire.up|cooldown.storm_earth_and_fire.charges=2)
    -- actions+=/call_action_list,name=sef,if=(!talent.serenity.enabled&cooldown.fists_of_fury.remains<=12&chi>=3&cooldown.rising_sun_kick.remains<=1)|target.time_to_die<=25|cooldown.touch_of_death.remains>112
    -- actions+=/call_action_list,name=sef,if=(!talent.serenity.enabled&!equipped.drinking_horn_cover&cooldown.fists_of_fury.remains<=6&chi>=3&cooldown.rising_sun_kick.remains<=1)|target.time_to_die<=15|cooldown.touch_of_death.remains>112&cooldown.storm_earth_and_fire.charges=1
    -- actions+=/call_action_list,name=sef,if=(!talent.serenity.enabled&cooldown.fists_of_fury.remains<=12&chi>=3&cooldown.rising_sun_kick.remains<=1)|target.time_to_die<=25|cooldown.touch_of_death.remains>112&cooldown.storm_earth_and_fire.charges=1
    function self.Use()
        return (not Talent.Serenity.Enabled() and (Player.Buff(Aura.StormEarthAndFire).Up() or Spell.StormEarthAndFire.Charges() == 2))
            or ((not Talent.Serenity.Enabled() and Spell.FistsOfFury.Cooldown.Remains() <= 12 and Player.Chi() >= 3 and Spell.RisingSunKick.Cooldown.Remains() <= 1) or Target.TimeToDie() <= 25 or Spell.TouchOfDeath.Cooldown.Remains() > 112)
            or (((not Talent.Serenity.Enabled() and Spell.FistsOfFury.Cooldown.Remains() <= 6 and Player.Chi() >= 3 and Spell.RisingSunKick.Cooldown.Remains() <= 1) or Target.TimeToDie() <= 15 or Spell.TouchOfDeath.Cooldown.Remains() > 112) and Spell.StormEarthAndFire.Charges() == 1)
            or (((not Talent.Serenity.Enabled() and Spell.FistsOfFury.Cooldown.Remains() <= 12 and Player.Chi() >= 3 and Spell.RisingSunKick.Cooldown.Remains() <= 1) or Target.TimeToDie() <= 25 or Spell.TouchOfDeath.Cooldown.Remains() > 112) and Spell.StormEarthAndFire.Charges() == 1)
    end
    return self;
end

-- Create a variable so we can call the rotations functions.
local rotationStormEarthAndFire = StormEarthAndFire("StormEarthAndFire");

-- Base APL Class
local function APL(rotationName, rotationDescription, specID)
    -- Inherits APL Class so get the base class.
    local self = addonTable.rotationsAPL(rotationName, rotationDescription, specID);

    -- Store the information for the script.
    self.scriptInfo = {
        SpecializationID = addonTable.MONK_WINDWALKER,
        ScriptAuthor = "HuntsTheWind",
        GuideAuthor = "Babylonius and SimCraft",
        GuideLink = "http://www.icy-veins.com/wow/windwalker-monk-pve-dps-guide",
        WoWVersion = 80000,
    };

    -- Set the preset builds for the script.
    self.presetBuilds = {
        ["Single Target"] = "3010032",
        ["AOE"] = "1010012",
    };

    self.Defensives = {
        DampenHarm = function()
            return Player.PhysicalDamagePredicted(5) >= 25;
        end,

        DiffuseMagic = function()
            return Player.MagicDamagePredicted(3) >= 30;
        end,

        TouchOfKarma = function()
            return Player.DamagePredicted(5) >= 25;
        end,
    };

    self.Interrupts = {
        ArcaneTorrent = function()
            return Target.InRange(8);
        end,

        LegSweep = function()
            return Target.InRange(5);
        end,

        WarStomp = function()
            return Target.InRange(5);
        end,
    };

    -- Table to hold requirements to use spells and items listed in the rotation.
    self.Requirements = {
        -- trinket.proc.agility.react can't be done without long lists of hard coded trinket data, so just skip trinket.proc conditions.
        -- actions+=/potion,if=buff.serenity.up|buff.storm_earth_and_fire.up|(!talent.serenity.enabled&trinket.proc.agility.react)|buff.bloodlust.react|target.time_to_die<=60
        ProlongedPower = function()
            return Player.Buff(Aura.Serenity).Up()
                or Player.Buff(Aura.StormEarthAndFire).Up()
                or not Talent.Serenity.Enabled()
                or Player.HasBloodlust()
                or Target.TimeToDie() <= 60;
        end,

        -- actions+=/touch_of_death,if=target.time_to_die<=9
        TouchOfDeath = function()
            return Target.TimeToDie() <= 9;
        end,
    };

    Objects.FinalizeRequirements(self.Defensives, self.Interrupts, self.Requirements);

    -- Function for setting up action objects such as spells, buffs, debuffs and items, called when the rotation becomes the active rotation.
    function self.Enable(...)
        -- Spells
        Racial, Spell, Talent, Aura, Azerite = ...; 

        Item = {};

        Consumable = {
            -- Potions
            ProlongedPower = Objects.newItem(142117),
        };

        Objects.FinalizeActions(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
    end

    -- Function for setting up the configuration screen, called when rotation becomes the active rotation.
    function self.SetupConfiguration(config, options)
        config.RacialOptions(options, Racial.ArcaneTorrent, Racial.Berserking, Racial.BloodFury, Racial.GiftOfTheNaaru, Racial.Shadowmeld);
        config.AOEOptions(options, Spell.FistsOfFury, Talent.RushingJadeWind, Spell.SpinningCraneKick, Talent.ChiBurst);
        config.CooldownOptions(options, Talent.ChiWave, Talent.EnergizingElixir, Talent.InvokeXuenTheWhiteTiger, Talent.Serenity, Spell.StormEarthAndFire, Spell.TouchOfDeath,
                                     Talent.WhirlingDragonPunch);
        config.DefensiveOptions(options, Talent.DampenHarm, Talent.DiffuseMagic, Spell.TouchOfKarma);
    end

    -- Function for destroying action objects such as spells, buffs, debuffs and items, called when the rotation is no longer the active rotation.
    function self.Disable()
        local coreTables = addon.Core.Tables;

        coreTables.WipeTables(Racial, Spell, Talent, Aura, Azerite, Item, Consumable);
    end

    -- Function for checking the rotation that displays on the Defensives icon.
    function self.Defensive(action)
        -- The abilities here should be listed from highest damage required to suggest to lowest,
        -- Specific damage types before all damage types.

        -- Redirect damage back to target
        action.EvaluateDefensiveAction(Spell.TouchOfKarma, self.Defensives.TouchOfKarma);

        -- Protects against magical damage
        action.EvaluateDefensiveAction(Talent.DiffuseMagic, self.Defensives.DiffuseMagic);

        -- Protects against physical damage
        action.EvaluateDefensiveAction(Talent.DampenHarm, self.Defensives.DampenHarm);
    end

    -- Function for displaying interrupts when target is casting an interruptible spell.
    function self.Interrupt(action)
        action.EvaluateInterruptAction(Spell.SpearHandStrike, true);
        action.EvaluateInterruptAction(Racial.ArcaneTorrent, self.Interrupts.ArcaneTorrent);

        -- Stuns
        if Target.IsStunnable() then
            action.EvaluateInterruptAction(Racial.QuakingPalm, true);
            action.EvaluateInterruptAction(Talent.LegSweep, self.Interrupts.LegSweep);
            action.EvaluateInterruptAction(Racial.WarStomp, self.Interrupts.WarStomp);
        end

        -- Crowd Control
        action.EvaluateInterruptAction(Spell.Paralysis, true);
    end

    -- Function for displaying opening rotation.
    function self.Opener(action)
    end

    -- Function for displaying any actions before combat starts.
    function self.Precombat(action)
        -- actions.precombat+=/potion
        action.EvaluateAction(Consumable.ProlongedPower, true);
        -- actions.precombat+=/chi_burst
        action.EvaluateAction(Talent.ChiBurst, true);
        -- actions.precombat+=/chi_wave
        action.EvaluateAction(Talent.ChiWave, true);
        -- in case player has chosen Eye of the Tiger at lvl 15
        action.EvaluateAction(Spell.TouchOfDeath, true);
    end

    -- Function for checking the rotation that displays on the Single Target, AOE, Off GCD and CD icons.
    function self.Combat(action)
        action.EvaluateAction(Consumable.ProlongedPower, self.Requirements.ProlongedPower);
        action.EvaluateAction(Spell.TouchOfDeath, self.Requirements.TouchOfDeath);

        action.CallActionList(rotationSerenity);
        action.CallActionList(rotationStormEarthAndFire);
        -- actions+=/call_action_list,name=aoe,if=active_enemies>3
        action.CallActionList(rotationAOE);
        -- actions+=/call_action_list,name=st,if=active_enemies<=3
        action.CallActionList(rotationStandard);
    end

    return self;
end

local rotationAPL = APL(nameAPL, "HuntsTheWind: Windwalker Monk", addonTable.Enum.SpecID.MONK_WINDWALKER);
