-- Pulls back the Addon-Local Variables and store them locally.
local _, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];

-- Plugin information variables.
local pluginName = "Equipment Manager";
local pluginDescription = "This plugin changes the players equipment set based on specialization.\n\nThis plugin will also ensure that the equipment set is kept updated when you equip new items such as boss drops without any interaction from the user!";

-- Check for LunaEclipse Rotation Helper being loaded first so it doesn't register the plugin if its not needed.
if addon then
    local optionsTable = {
        type = "group",
        name = pluginName,
        get = function(info)
            local returnValue = 0;

            if info then
                local Tables = addon.Core.Tables;
                local profile = addonTable.equipManager;
                local keyName = info[#info];

                returnValue = Tables.GetValue(profile, keyName);

                if returnValue == nil then
                    returnValue = 0;
                    profile[keyName] = returnValue;

                    addonTable.equipManager = profile;
                end

                if info.option.type == "toggle" then
                    returnValue = addon.Options.Core.ConvertToUI(returnValue, info.option.tristate);
                end
            end

            return returnValue;
        end,
        set = function(info, value)
            if info then
                local profile = addonTable.equipManager;
                local keyName = info[#info];

                if info.option.type == "toggle" then
                    value = addon.Options.Core.ConvertToValue(value, info.option.tristate);
                end

                profile[keyName] = value;

                -- Put the settings into the database
                addonTable.equipManager = profile;
            end
        end,
        childGroups = "tree",
        args = {
            useOverlay = {
                type = "toggle",
                name = "Use the Equipment Set Overlay",
                desc = "This will show missing items or items that will change as a color overlay when selecting an equipment set.",
                width = "full",
                order = 10,
            },
            disableDowngrade = {
                type = "toggle",
                name = "Disable Auto Save on item downgrade.",
                desc = "This will stop the equipment set from being saved if the item is considered a downgrade.\n\nEnabling this option is useful if you run old content for transmog items and need to equip them to collect the transmog.",
                width = "full",
                order = 20,
            },
            downgradeLevel = {
                type = "range",
                name = "Item level difference to be considered a downgrade.",
                desc = "This setting allows you to specify how much lower the iLevel must be on the item to be considered a downgrade.\n\nThis is because sometimes for raiding it's better to equip a lower level item because it contains better stats.\n\nUsing this setting you can equip an item with a lower iLevel and still have the equipment set automatically update.",
                min = 0,
                max = 50,
                step = 5,
                width = "full",
                order = 30,
            },
        },
    };

    -- The plugin still has to be registered even if its not loaded into memory, otherwise there is no checkbox to re-enable it.
    addon.System.Plugins.RegisterPlugin(pluginName, pluginDescription, optionsTable);
end