-- Pulls back the Addon-Local Variables and store them locally.
local _, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];

-- Plugin information variables.
local pluginName = "Minimap Icon/LDB Display";
local pluginDescription = "This plugin allows the user to access certain features via a minimap icon or LDB display.\n\nIf you are using this as an LDB display, it is recommended to hide the minimap icon!";

-- Check for LunaEclipse Rotation Helper being loaded first so it doesn't register the plugin if its not needed.
if addon then
    local optionsTable = {
        type = "group",
        name = pluginName,
        get = function(info)
            local returnValue = 1;

            if info then
                local Tables = addon.Core.Tables;
                local profile = addonTable.minimapSettings;
                local keyName = info[#info];

                returnValue = Tables.GetValue(profile, keyName);

                if returnValue == nil then
                    returnValue = 1;
                    profile[keyName] = returnValue;

                    addonTable.minimapSettings = profile;
                end

                if info.option.type == "toggle" then
                    returnValue = addon.Options.Core.ConvertToUI(returnValue, info.option.tristate);
                end
            end

            return returnValue;
        end,
        set = function(info, value)
            if info then
                local profile = addonTable.minimapSettings;
                local keyName = info[#info];

                if info.option.type == "toggle" then
                    value = addon.Options.Core.ConvertToValue(value, info.option.tristate);
                end

                profile[keyName] = value;

                -- Put the settings into the database
                addonTable.minimapSettings = profile;

                addon.System.Callbacks.Callback("updateMinimapIcon");
            end
        end,
        childGroups = "tree",
        args = {
            minimapOptions = {
                type = "group",
                name = addon.Options.Core.CreaterHeader("Standard Minimap Options"),
                order = 10,
                inline = true,
                args = {
                    hide = {
                        type = "toggle",
                        name = "Hide Minimap Icon",
                        desc = "This option allows you to disable the minimap icon and use this as an LDB display for ElvUI, Titan Panel, etc...",
                        get = function(info)
                            local returnValue = false;

                            if info then
                                local Tables = addon.Core.Tables;
                                local profile = addonTable.minimapSettings;
                                local keyName = info[#info];

                                returnValue = Tables.GetValue(profile, keyName);
                            end

                            return returnValue;
                        end,
                        set = function(info, value)
                            if info then
                                addon.System.Callbacks.Callback("toggleMinimap");
                            end
                        end,
                        width = "full",
                        order = 10,
                    },
                    useRotations = {
                        type = "toggle",
                        name = "Integrate Changing Rotations (Left Click)",
                        desc = "This option adds the ability to change rotations by left clicking when there is more than one rotation registered for your specialization.",
                        width = "full",
                        order = 20,
                    },
                    useTalentBuilds = {
                        type = "toggle",
                        name = "Integrate Talent Build Plugin (Right Click)",
                        desc = "This option adds Talent Build plugin information to the tooltip and mouse click options.",
                        hidden = function()
                            return not addon.System.Plugins.IsActive("Talent Builds");
                        end,
                        width = "full",
                        order = 30,
                    },
                },
            },
            modifierControl = {
                type = "group",
                name = addon.Options.Core.CreaterHeader("Control key modifier Minimap Options"),
                order = 30,
                inline = true,
                args = {
                    changeSpec = {
                        type = "toggle",
                        name = "Integrate Changing Specialization (Control + Left Click)",
                        desc = "This option adds the ability to change your current specialization by left clicking while holding the Control key.",
                        width = "full",
                        order = 10,
                    },
                    changeLootSpec = {
                        type = "toggle",
                        name = "Integrate Changing Loot Specialization (Control + Right Click)",
                        desc = "This option adds the ability to change your current loot specialization by right clicking while holding the Control key.",
                        width = "full",
                        order = 20,
                    },
                },
            },
            modifierShift = {
                type = "group",
                name = addon.Options.Core.CreaterHeader("Shift key modifier Minimap Options"),
                order = 40,
                inline = true,
                args = {
                    toggleIcon = {
                        type = "toggle",
                        name = "Toggle Minimap Icon On/Off (Shift + Left Click)",
                        desc = "This option allows you to toggle the minimap icon visibility by left clicking while holding the Shift key.",
                        width = "full",
                        order = 10,
                    },
                    reloadUI = {
                        type = "toggle",
                        name = "Integrate UI Reload (Shift + Right Click)",
                        desc = "This option adds the ability to reload your UI by right clicking while holding the Shift key.",
                        width = "full",
                        order = 20,
                    },
                },
            },
        },
    };

    -- The plugin still has to be registered even if its not loaded into memory, otherwise there is no checkbox to re-enable it.
    addon.System.Plugins.RegisterPlugin(pluginName, pluginDescription, optionsTable);
end
