-- Pulls back the Addon-Local Variables and store them locally.
local _, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];

-- Plugin information variables.
local pluginName = "Talent Builds";
local pluginDescription = "This plugin allows the user to access the preset talent builds associated with a rotation.\n\nIt also allows the user to modify these talent builds or create their own!";

-- Check for LunaEclipse Rotation Helper being loaded first so it doesn't register the plugin if its not needed.
if addon then
    local optionsTable = {
        type = "group",
        name = pluginName,
        get = function(info)
            local returnValue = 0;

            if info then
                local Tables = addon.Core.Tables;
                local profile = addonTable.addonSettings;
                local keyName = info[#info];

                returnValue = Tables.GetValue(profile, keyName);

                if returnValue == nil then
                    returnValue = 0;
                    profile[keyName] = returnValue;

                    addonTable.addonSettings = profile;
                end
            end

            return addon.Options.Core.ConvertToUI(returnValue, info.option.tristate);
        end,
        set = function(info, value)
            if info then
                local profile = addonTable.addonSettings;
                local keyName = info[#info];

                if info.option.type == "toggle" then
                    value = addon.Options.Core.ConvertToValue(value, info.option.tristate);
                end

                profile[keyName] = value;

                -- Put the settings into the database
                addonTable.addonSettings = profile;
            end
        end,
        childGroups = "tree",
        args = {
            autoLearnTalents = {
                type = "toggle",
                name = "Automatically learn talents from the selected build.",
                desc = "This will automatically learn new talents from your selected talent build when you have unspent talent points available.\n\nIf this is unchecked you will be prompted if you want to learn the appropriate talents.",
                width = "full",
                order = 10,
            },
        },
    };

    -- The plugin still has to be registered even if its not loaded into memory, otherwise there is no checkbox to re-enable it.
    addon.System.Plugins.RegisterPlugin(pluginName, pluginDescription, optionsTable);
end
