-- Pulls back the Addon-Local Variables and store them locally.
local addonName, addonTable = ...;
local addon = _G["LunaEclipse_Rotation"];

-- Check for LunaEclipse Rotation Helper being loaded first, there is no point doing anything else if its not loaded.
if addon then
	local defaultAddonSettings = {
		allowZeroMacros = 0,
		autoLearnTalents = 0,
		styleSelection = "Ovale",
		versionWeakAura = 0,
	};

	local defaultCharacterSettings = {
		versionMacroProfile = 0,
	};

	local defaultEquipManager = {
		useOverlay = 1,
		disableDowngrade = 1,
		downgradeLevel = 25,
		Sets = {},
	};

	local defaultMacroProfile = {};
	local defaultMinimapSettings = {
		changeLootSpec = 1,
		changeSpec = 1,
		hide = false,
		reloadUI = 1,
		toggleIcon = 1,
		useRotations = 1,
		useTalentBuilds = 1,
	};

	local defaultTalentBuilds = {
		CurrentBuild = {},
		CustomBuilds = {},
	};
	local defaultWeakAuraSettings = {
		CustomAuras = {},
		Styles = {
			Hekili = {
				LERH_CD = 1,
				LERH_DamageAll = 0,
				LERH_DamageMagic = 0,
				LERH_DamagePhysical = 0,
				LERH_Defensive = 1,
				LERH_Enemies = 0,
				LERH_Interrupt = 1,
				LERH_OffGCD = 1,
				LERH_PRIORITY_1 = 1,
				LERH_PRIORITY_2 = 1,
				LERH_PRIORITY_3 = 1,
				LERH_PRIORITY_4 = 1,
				LERH_Range = 0,
				LERH_TimeToDie = 0,
			},
			LunaEclipse = {
				LERH_AOE = 1,
				LERH_CD = 1,
				LERH_DamageAll = 0,
				LERH_DamageMagic = 0,
				LERH_DamagePhysical = 0,
				LERH_Defensive = 1,
				LERH_Enemies = 0,
				LERH_Interrupt = 1,
				LERH_OffGCD = 1,
				LERH_Range = 0,
				LERH_TimeToDie = 0,
			},
			Ovale = {
				LERH_AOE = 1,
				LERH_CD = 1,
				LERH_DamageAll = 0,
				LERH_DamageMagic = 0,
				LERH_DamagePhysical = 0,
				LERH_Defensive = 1,
				LERH_Enemies = 0,
				LERH_Interrupt = 1,
				LERH_OffGCD = 1,
				LERH_Range = 0,
				LERH_ST = 1,
				LERH_TimeToDie = 0,
			},
		},
	};

	-- Color Tags
	addonTable.COLORS = {
		CLASS = {
			[1] = "|cFFC79C6E",
			[2] = "|cFFF58CBA",
			[3] = "|cFFABD473",
			[4] = "|cFFFFF569",
			[5] = "|cFFFFFFFF",
			[6] = "|cFFC41F3B",
			[7] = "|cFF0070DE",
			[8] = "|cFF69CCF0",
			[9] = "|cFF9482C9",
			[10] = "|cFF00FF96",
			[11] = "|cFFFF7D0A",
			[12] = "|cFFA330C9",
		},

		TAGS = {
			ACTIVE = "|cFF00FF00",
			DEFAULT = "|cFF7FCCFF",
			HIGHLIGHT = "|cFF1784D1",
			INACTIVE = "|cFFFF0000",
			MESSAGE = "|cFFFFFF00",
		},
	};

	-- General Tags
	addonTable.TAGS = {
		END = "|r",
	};

	-- Table for holding Talent Build data.
	addonTable.TalentBuilds = {
		Custom = "Custom Build",
		MaxLevel = 110,
	};

	-- Internal namespace for private WeakAuras functions and data.
	addonTable.WeakAuras = {
		Core = {},
		Styles = {},
		Templates = {},
		Version = 4,
	};

	-- Tables to hold private functions.
	local profileVersion = 4;
	local variableError = false;

	local moduleEvents = {
		addonLoaded = function(name)
			if name == addonName then
				local Tables = addon.Core.Tables;

				addonTable.addonSettings = Tables.ApplyDefaults(LERH_Features_AddonSettings, defaultAddonSettings);
				addonTable.characterSettings = Tables.ApplyDefaults(LERH_Features_CharacterSettings, defaultCharacterSettings);
				addonTable.equipManager = Tables.ApplyDefaults(LERH_EquipManager, defaultEquipManager);
				addonTable.macroProfiles = Tables.ApplyDefaults(LERH_MacroProfiles, defaultMacroProfile);
				addonTable.minimapSettings = Tables.ApplyDefaults(LERH_Minimap, defaultMinimapSettings);
				addonTable.talentBuilds = Tables.ApplyDefaults(LERH_TalentBuilds, defaultTalentBuilds);
				addonTable.weakAuraSettings = Tables.ApplyDefaults(LERH_WeakAuras, defaultWeakAuraSettings);

				if LESM_MacroProfile and Tables.IsEmpty(addonTable.macroProfiles) then
					addonTable.macroProfiles = LESM_MacroProfile.profiles.Settings;
				end

				if addon.System.Plugins.IsActive("Macro Manager") and Tables.GetValue(addonTable.characterSettings, "versionMacroProfile", 0) < profileVersion then
					addonTable.macroProfiles = addon.Features.MacroManager.Update(addonTable.macroProfiles);
					addonTable.characterSettings.versionMacroProfile = profileVersion;
				end
			end
		end,

		errorSavedVariables = function(name)
			if name == addonName then
				variableError = true;
			end
		end,

		playerLogout = function()
			if not variableError then
				LERH_Features_AddonSettings = addonTable.addonSettings;
				LERH_Features_CharacterSettings = addonTable.characterSettings;
				LERH_EquipManager = addonTable.equipManager;
				LERH_MacroProfiles = addonTable.macroProfiles;
				LERH_Minimap = addonTable.minimapSettings;
				LERH_TalentBuilds = addonTable.talentBuilds;
				LERH_WeakAuras = addonTable.weakAuraSettings;
			end
		end,
	};

	addon.System.Callbacks.RegisterAddonCallbacks(moduleEvents);
end